@@ -23,7 +23,6 @@
 #include "toolcontext.h"
 #include "lvm-file.h"
 #include "memlock.h"
-#include "segtype.h"
 
 #include <sys/stat.h>
 #include <sys/mman.h>
@@ -892,7 +891,7 @@ static int _check_value_differs_from_default(struct cft_check_handle *handle,
 				}
 				break;
 			case DM_CFG_EMPTY_ARRAY:
-				diff = v_def->type != DM_CFG_EMPTY_ARRAY;
+				diff = (v_def && (v_def->type != DM_CFG_EMPTY_ARRAY));
 				break;
 			default:
 				log_error(INTERNAL_ERROR "inconsistent state reached in _check_value_differs_from_default");
@@ -1216,7 +1215,7 @@ const struct dm_config_node *find_config_tree_node(struct cmd_context *cmd, int
 
 	cn = dm_config_tree_find_node(cmd->cft, path);
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return cn;
@@ -1238,7 +1237,7 @@ const char *find_config_tree_str(struct cmd_context *cmd, int id, struct profile
 	str = _config_disabled(cmd, item, path) ? cfg_def_get_default_value(cmd, item, CFG_TYPE_STRING, profile)
 						: dm_config_tree_find_str(cmd->cft, path, cfg_def_get_default_value(cmd, item, CFG_TYPE_STRING, profile));
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return str;
@@ -1262,7 +1261,7 @@ const char *find_config_tree_str_allow_empty(struct cmd_context *cmd, int id, st
 	str = _config_disabled(cmd, item, path) ? cfg_def_get_default_value(cmd, item, CFG_TYPE_STRING, profile)
 						: dm_config_tree_find_str_allow_empty(cmd->cft, path, cfg_def_get_default_value(cmd, item, CFG_TYPE_STRING, profile));
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return str;
@@ -1284,7 +1283,7 @@ int find_config_tree_int(struct cmd_context *cmd, int id, struct profile *profil
 	i = _config_disabled(cmd, item, path) ? cfg_def_get_default_value(cmd, item, CFG_TYPE_INT, profile)
 					      : dm_config_tree_find_int(cmd->cft, path, cfg_def_get_default_value(cmd, item, CFG_TYPE_INT, profile));
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return i;
@@ -1306,7 +1305,7 @@ int64_t find_config_tree_int64(struct cmd_context *cmd, int id, struct profile *
 	i64 = _config_disabled(cmd, item, path) ? cfg_def_get_default_value(cmd, item, CFG_TYPE_INT, profile)
 						: dm_config_tree_find_int64(cmd->cft, path, cfg_def_get_default_value(cmd, item, CFG_TYPE_INT, profile));
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return i64;
@@ -1328,7 +1327,7 @@ float find_config_tree_float(struct cmd_context *cmd, int id, struct profile *pr
 	f = _config_disabled(cmd, item, path) ? cfg_def_get_default_value(cmd, item, CFG_TYPE_FLOAT, profile)
 					      : dm_config_tree_find_float(cmd->cft, path, cfg_def_get_default_value(cmd, item, CFG_TYPE_FLOAT, profile));
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return f;
@@ -1367,7 +1366,7 @@ int find_config_tree_bool(struct cmd_context *cmd, int id, struct profile *profi
 	b = _config_disabled(cmd, item, path) ? cfg_def_get_default_value(cmd, item, CFG_TYPE_BOOL, profile)
 					      : dm_config_tree_find_bool(cmd->cft, path, cfg_def_get_default_value(cmd, item, CFG_TYPE_BOOL, profile));
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return b;
@@ -1467,7 +1466,7 @@ const struct dm_config_node *find_config_tree_array(struct cmd_context *cmd, int
 		cn = cn_def;
 	}
 
-	if (profile_applied)
+	if (profile_applied && profile)
 		remove_config_tree_by_source(cmd, profile->source);
 
 	return cn;
@@ -1695,6 +1694,11 @@ static int _out_prefix_fn(const struct dm_config_node *cn, const char *line, voi
 		if (cfg_def->comment) {
 			int pos = 0;
 			while (_copy_one_line(cfg_def->comment, commentline, &pos, strlen(cfg_def->comment))) {
+				if ((commentline[0] == '#') && (strlen(commentline) == 1)) {
+					if (!out->tree_spec->withspaces)
+						continue;
+					commentline[0] = '\0';
+				}
 				fprintf(out->fp, "%s# %s\n", line, commentline);
 				/* withsummary prints only the first comment line. */
 				if (!out->tree_spec->withcomments)
@@ -1717,6 +1721,9 @@ static int _out_prefix_fn(const struct dm_config_node *cn, const char *line, voi
 		if (cfg_def->flags & CFG_DEFAULT_UNDEFINED)
 			fprintf(out->fp, "%s# This configuration %s does not have a default value defined.\n", line, node_type_name);
 
+		if (cfg_def->flags & CFG_DEFAULT_COMMENTED)
+			fprintf(out->fp, "%s# This configuration %s has an automatic default value.\n", line, node_type_name);
+
 		if ((out->tree_spec->type == CFG_DEF_TREE_FULL) &&
 		    (out->tree_spec->check_status[cn->id] & CFG_USED))
 			fprintf(out->fp, "%s# Value defined in existing configuration has been used for this setting.\n", line);
