@@ -16,7 +16,9 @@
 #   instead lvconvert --repair them?)
 # - linear LVs with bits missing are not activated
 
-. lib/test
+SKIP_WITH_LVMPOLLD=1
+
+. lib/inittest
 
 aux prepare_vg 2
 
