@@ -11,9 +11,9 @@
 
 # Exercise usage of stacked cache volume used in thin pool volumes
 
-. lib/inittest
+SKIP_WITH_LVMPOLLD=1
 
-test -e LOCAL_LVMPOLLD && skip
+. lib/inittest
 
 aux have_cache 1 3 0 || skip
 aux have_thin 1 0 0 || skip
